// Better random number generator from Numerical Recipes in C

#include <iostream.h>
#include "ran1.h"

Ran1::Ran1() {
  iy = 0;
  idum = -1;
  iset = 0;

  setup();
}

void Ran1::setup() {
  if (-idum < 1) idum = 1;
  else idum = -idum;

  int j;
  for (j = NTAB+7; j >= 0; j--) {
    long k = idum/IQ;
    idum = IA * (idum - k*IQ) - IR*k;
    if (idum < 0) idum += IM;
    if (j < NTAB) iv[j] = idum;
  }
  iy = iv[0];
}

// Return uniformly distributed double on (0,1)
double Ran1::get_num() {
  if (!iy)
    setup();

  long k = idum/IQ;
  idum = IA * (idum - k*IQ) - IR*k;
  if (idum < 0) idum += IM;
  int j = iy/NDIV;
  iy = iv[j];
  iv[j] = idum;
  double temp;
  if ((temp = AM*iy) > RNMX) return RNMX;
  else return temp;
}

// Return 0 mean, unit variance gaussian
double Ran1::gaussian() {
  double fac, rsq, v1, v2;

  if (iset == 0) {
    do {
      v1 = 2.0 * get_num() - 1.0;
      v2 = 2.0 * get_num() - 1.0;
      rsq = v1*v1 + v2*v2;
    } while (rsq >= 1.0 || rsq == 0.0);
    fac = sqrt(-2.0 * log(rsq) / rsq);
    gset = v1 * fac;
    iset = 1;
    return v2 * fac;
  }
  else {
    iset = 0;
    return gset;
  }
}
