//=============================================================================
//
// speeders.cc	Do speed tests for possible implementations
//
// Author:	Peter Stubley
// Created:	Fri Nov 15 1996
//
// Copyright:	Nortel Technology, 1996
//
//=============================================================================

#include <iostream.h>

#include <watch.h>
#include <ran1.h>

int main()
{
  mdl_watch timer;

  // Build the data
  double *source = new double[1000];
  double *dists = new double[1000];
  int *starts = new int[1000];
  double *sc = new double[1000];

  // Fill source and dists with data
  Ran1 ran;
  for ( int t = 0; t < 1000; t++ ) {
    source[t] = ran.get_num();
    dists[t] = ran.get_num();
  }

  // Basic version - no start times
  double *dest = new double[1000];

  timer.reset();
  timer.start();
  for ( int it = 0; it < 100000; it++ ) {
    double s1 = -1e20;
    for ( int t = 0; t < 1000; t++ ) {
      dest[t] = s1 + dists[t];

      if ( source[t] > s1 ) {
	s1 = source[t];
	starts[t] = t;
	//sc[t] = source[t];
      }

      s1 += dists[t];
    }
  }
  timer.stop();
  cout << "Type 1 (basic, doubles): " << timer.user() << endl;

  // Build the data
  float *fsource = new float[1000];
  float *fdists = new float[1000];
  float *fsc = new float[1000];

  // Fill source and dists with data
  Ran1 fran;
  for ( t = 0; t < 1000; t++ ) {
    fsource[t] = fran.get_num();
    fdists[t] = fran.get_num();
  }

  // Basic version - no start times
  float *fdest = new float[1000];

  timer.reset();
  timer.start();
  for ( int fit = 0; fit < 100000; fit++ ) {
    float s1 = -1e20;
    for ( int t = 0; t < 1000; t++ ) {
      fdest[t] = s1 + fdists[t];

      if ( fsource[t] > s1 ) {
	s1 = fsource[t];
	starts[t] = t;
	//fsc[t] = fsource[t];
      }

      s1 += fdists[t];
    }
  }
  timer.stop();
  cout << "Type 2 (basic, floats): " << timer.user() << endl;

  delete[] source;
  delete[] dists;
  delete[] dest;
  delete[] fsource;
  delete[] fdists;
  delete[] fdest;

  return 0;
}
