/*------------------------------------------------------------------
 *
 *         watch.cc
 *
 *         watch.cc,v 1.5 1994/11/11 21:41:41 chris Exp
 *
 *  watch.cc,v
// Revision 1.5  1994/11/11  21:41:41  chris
// New name for STring Class and minor fluff
//
// Revision 1.4  1994/04/17  03:08:51  chris
// ootp20.1 sources
//
 *
 *------------------------------------------------------------------*/
/* watch.cc */
#ifdef SYSVR4_88K
#include <unistd.h>
#endif
#include <sys/param.h> 
#include <sys/times.h>


#ifdef SUN_SPARC
#include <sys/time.h>
#endif

#ifndef HZ
#ifdef CLK_TCK
#define HZ CLK_TCK
#else
#define HZ 60
#endif
#endif


#include "watch.h"

#ifdef SUN_SPARC
static long tv_sec_base = 0;
#endif

/*---------------------------------------------------------------------
 *
 * mdl_watch::mdl_watch()
 *
 *-------------------------------------------------------------------*/
mdl_watch::mdl_watch()
{
  state = stopped;
#ifdef SUN_SPARC
  if( tv_sec_base == 0 )
    {
      struct timeval now;
      struct timezone z;
      gettimeofday(&now,&z);
      tv_sec_base = now.tv_sec;
    }
#endif
  reset();

};

/*---------------------------------------------------------------------
 *
 * void mdl_watch::reset()
 *
 *-------------------------------------------------------------------*/
void mdl_watch::reset()
{
  if( state == running )
    {
      tms buf;
      real_time   = -times( &buf );
      user_time   = -buf.tms_utime;
      system_time = -buf.tms_stime;
#ifdef SUN_SPARC
      struct timeval now;
      struct timezone z;
      gettimeofday(&now,&z);
      real_time = -((now.tv_sec-tv_sec_base)*HZ+now.tv_usec*HZ/1000000);
#endif
    }
  else
    {
      real_time = 0;
      user_time = 0;
      system_time = 0;
    }
}

/*---------------------------------------------------------------------
 *
 * void mdl_watch::start()
 *
 *-------------------------------------------------------------------*/
void mdl_watch::start()
{
  if( state == stopped )
    {
      tms buf;
      real_time -= times( &buf );
      user_time -= buf.tms_utime;
      system_time -= buf.tms_stime;
#ifdef SUN_SPARC
      struct timeval now;
      struct timezone z;
      gettimeofday(&now,&z);
      real_time = -((now.tv_sec-tv_sec_base)*HZ+now.tv_usec*HZ/1000000);
#endif
      state = running;
    }
}

/*---------------------------------------------------------------------
 *
 * void mdl_watch::stop()
 *
 *-------------------------------------------------------------------*/
void mdl_watch::stop()
{
  if( state == running )
    {
      tms buf;
      real_time += times( &buf );
      user_time += buf.tms_utime;
      system_time += buf.tms_stime; 
#ifdef SUN_SPARC
      struct timeval now;
      struct timezone z;
      gettimeofday(&now,&z);
      real_time += ((now.tv_sec-tv_sec_base)*HZ+now.tv_usec*HZ/1000000);
#endif
      state = stopped;
    }
}
/*---------------------------------------------------------------------
 *
 * double mdl_watch::user() const 
 *
 *-------------------------------------------------------------------*/
double mdl_watch::user() const 
{
  if( state == stopped )
    return double(user_time)/double(HZ);

  tms buf;
  times( &buf );
  return double(  buf.tms_utime + user_time )/double(HZ);
}

/*---------------------------------------------------------------------
 *
 * double mdl_watch::system() const 
 *
 *-------------------------------------------------------------------*/
double mdl_watch::system() const 
{
  if( state == stopped )
    return double(system_time)/double(HZ);

  tms buf;
  times( &buf );
  return double( buf.tms_stime + system_time )/double(HZ);
}

/*---------------------------------------------------------------------
 *
 * double mdl_watch::real() const 
 *
 *-------------------------------------------------------------------*/
double mdl_watch::real() const 
{
  if( state == stopped )
    return double(real_time)/double(HZ);

#ifndef SUN_SPARC
  tms buf;
  long r = times(&buf);
#else
  struct timeval now;
  struct timezone z;
  gettimeofday(&now,&z);
  long r = ((now.tv_sec-tv_sec_base)*HZ+now.tv_usec*HZ/1000000);
#endif

  return double( r + real_time )/double(HZ);
}

